local bn_helpers = require("BattleNetwork.Assets")

local sound = bn_helpers.load_audio("dust_launch.ogg")
local hit_sound = bn_helpers.load_audio("explosion_defeatedboss.ogg")
local meteor_texture = Resources.load_texture("meteor.png")
local boom_texture = Resources.load_texture("ring_explosion.png")

function card_init(player, props)
    local action = Action.new(player, "CHARACTER_THROW")
    action.on_execute_func = function(self, user)
        local hit_props = HitProps.new(
            props.damage,
            props.hit_flags,
            props.element,
            user:context(),
            Drag.None
        )
        create_meteor_component(player, hit_props) --Create the component, this will inject it as well.
    end
    return action
end

function create_meteor_component(player, props)
    if not player or player and player:deleted() then return end

    local meteor_component = player:create_component(Lifetime.ActiveBattle)
    meteor_component.count = math.random(3, 6)
    meteor_component.attack_cooldown_max = 32
    meteor_component.highlight_cooldown_max = 24
    meteor_component.highlight_cooldown = 24
    meteor_component.initial_cooldown = 40
    meteor_component.attack_cooldown = 0
    meteor_component.owner = player
    meteor_component.accuracy_chance = 20
    meteor_component.tile_list = {}
    meteor_component.create_once = true
    meteor_component.next_tile = nil
    meteor_component.desired_cooldown = 0
    meteor_component.props = props
    meteor_component.field = player:field()
    meteor_component.stored_team = player:team()
    meteor_component.enemy_filter = function(character)
        return character:team() ~= meteor_component.stored_team
    end
    for x = 1, 6, 1 do
        for y = 1, 3, 1 do
            local desired_tile = meteor_component.field:tile_at(x, y)
            --Neat trick: if you only have one declaration, you can put it all in a single line like this with the if statement.
            if not desired_tile or desired_tile and not desired_tile:is_edge() and not player:is_team(desired_tile:team()) then
                table.insert(meteor_component.tile_list, desired_tile)
            end
        end
    end
    meteor_component.on_update_func = function(self)
        if self.owner:deleted() then return end
        if self.count <= 0 then
            self:eject()
        else
            if self.initial_cooldown > 0 then
                self.initial_cooldown = self.initial_cooldown - 1
                return
            end
            if self.next_tile ~= nil then
                self.next_tile:set_highlight(Highlight.Flash)
            end
            if self.highlight_cooldown <= 0 then
                --Use less than or equal to copmarison to confirm a d100 roll of accuracy.
                --Example: if a player has an accuracy chance of 20, then a 1 to 100 roll will
                --Only target the player's tile on a roll of 1-20, leading to an 80% chance of
                --Targeting a random player tile.
                if math.random(1, 100) <= self.accuracy_chance then
                    local target_list = self.field:find_nearest_characters(self.owner, self.enemy_filter)
                    local target = nil
                    if #target_list > 0 then target = target_list[1] end
                    if target ~= nil then
                        self.next_tile = target:current_tile()
                    else
                        self.next_tile = self.tile_list[math.random(1, #self.tile_list)]
                    end
                else
                    self.next_tile = self.tile_list[math.random(1, #self.tile_list)]
                end
                self.highlight_cooldown = self.highlight_cooldown_max
            else
                self.highlight_cooldown = self.highlight_cooldown - 1
            end
            if self.attack_cooldown <= self.desired_cooldown and self.next_tile ~= nil then
                self.count = self.count - 1
                self.attack_cooldown_max = self.attack_cooldown_max
                self.attack_cooldown = self.attack_cooldown_max
                self.field:spawn(create_meteor(self.owner, self.props), self.next_tile)
            else
                self.attack_cooldown = self.attack_cooldown - 1
            end
        end
    end
end

function create_meteor(player, props)
    if not player or player and player:deleted() then return end
    local x = 224
    local y = 224

    local meteor = Spell.new(player:team())

    meteor:set_tile_highlight(Highlight.Flash)
    meteor:set_facing(player:facing())
    meteor:set_hit_props(props)

    meteor:set_texture(meteor_texture)

    local anim = meteor:animation()
    anim:load("meteor.animation")
    anim:set_state("DEFAULT")
    meteor:sprite():set_layer(-2)

    meteor.field = player:field()
    meteor.boom = boom_texture

    meteor.cooldown = 16

    meteor.increment_x = 14
    meteor.increment_y = 14

    if player:facing() == Direction.Right then x = x * -1 end

    meteor:set_offset(meteor:offset().x + x * 0.5, meteor:offset().y - y * 0.5)
    meteor.on_update_func = function(self)
        if self.cooldown <= 0 then
            local tile = self:current_tile()
            if tile and tile:is_walkable() then
                tile:attack_entities(self)
                self.field:shake(5, 18)
                local explosion = Spell.new(self:team())
                explosion:set_texture(self.boom)
                local new_anim = explosion:animation()
                new_anim:load("ring_explosion.animation")
                new_anim:set_state("DEFAULT")
                explosion:sprite():set_layer(-2)
                Resources.play_audio(sound)
                self.field:spawn(explosion, tile)
                new_anim:on_frame(1, function()
                    tile:set_state(TileState.Ice)
                end)
                new_anim:on_frame(3, function()
                    Resources.play_audio(hit_sound)
                end)
                new_anim:on_complete(function()
                    explosion:erase()
                end)
            end
            self:erase()
        else
            local offset = self:offset()
            if self:facing() == Direction.Right then
                self:set_offset(offset.x + self.increment_x * 0.5, offset.y + self.increment_y * 0.5)
            else
                self:set_offset(offset.x - self.increment_x * 0.5, offset.y + self.increment_y * 0.5)
            end
            self.cooldown = self.cooldown - 1
        end
    end
    meteor.can_move_to_func = function(tile)
        return true
    end
    return meteor
end
